/**
 * OW2 Util
 * Copyright (C) 2006-2012 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: EjbJarMethodMetadataConfigurator.java 6271 2012-07-10 11:10:15Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.ejbjar.impl.configurator;

import org.ow2.util.ee.deploy.api.deployable.EJBDeployable;
import org.ow2.util.ee.metadata.common.impl.configurator.CommonMethodMetadataConfigurator;
import org.ow2.util.ee.metadata.common.impl.configurator.visitor.JavaxAnnotationSecurityPermitAllVisitor;
import org.ow2.util.ee.metadata.common.impl.configurator.visitor.JavaxAnnotationSecurityRolesAllowedVisitor;
import org.ow2.util.ee.metadata.common.impl.configurator.visitor.JavaxEjbTransactionAttributeVisitor;
import org.ow2.util.ee.metadata.common.impl.configurator.visitor.JavaxInterceptorExcludeDefaultInterceptorsVisitor;
import org.ow2.util.ee.metadata.common.impl.configurator.visitor.JavaxInterceptorInterceptorsVisitor;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarClassMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarDeployableMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarFieldMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarMethodMetadata;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.JavaxEjbAccessTimeoutVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.JavaxEjbLockVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.method.JavaxAnnotationSecurityDenyAllVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.method.JavaxEjbAfterBeginVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.method.JavaxEjbAfterCompletionVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.method.JavaxEjbBeforeCompletionVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.method.JavaxEjbInitVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.method.JavaxEjbPostActivateVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.method.JavaxEjbPrePassivateVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.method.JavaxEjbRemoveVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.method.JavaxEjbScheduleVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.method.JavaxEjbSchedulesVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.method.JavaxEjbTimeoutVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.method.JavaxInterceptorAroundInvokeVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.method.JavaxInterceptorExcludeClassInterceptorsVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.method.JavaxJwsWebMethodVisitor;

/**
 * EjbJarMethodMetadataConfigurator.
 * @param <E> an implementation of EJBDeployable
 * @param <D> an implementation of IEjbJarDeployableMetadata
 * @param <C> an implementation of IEjbJarClassMetadata
 * @param <M> an implementation of IEjbJarMethodMetadata
 * @param <F> an implementation of IEjbJarFieldMetadata
 * @author Gael Lalire
 */
public class EjbJarMethodMetadataConfigurator<
    E extends EJBDeployable<E>,
    D extends IEjbJarDeployableMetadata<E, D, C, M, F>,
    C extends IEjbJarClassMetadata<E, D, C, M, F>,
    M extends IEjbJarMethodMetadata<E, D, C, M, F>,
    F extends IEjbJarFieldMetadata<E, D, C, M, F>>
        extends CommonMethodMetadataConfigurator<C, M, F> {

    /**
     * Constructor.
     * @param methodMetadata the method metadata
     * @param annotationParsingDesactived annotationParsingDesactived
     */
    public EjbJarMethodMetadataConfigurator(final M methodMetadata,
            final boolean annotationParsingDesactived) {
        super(methodMetadata, null);

        if (!annotationParsingDesactived) {
            registerAnnotationVisitor();
        }

    }

    /**
     * register annotation visitor.
     */
    @Override
    protected void registerAnnotationVisitor() {

        super.registerAnnotationVisitor();

        // TODO gaellalire : use interfaces in visitor
        M ejbJarMethodMetadata = getMethodMetadata();

        // add @Init
        getAnnotationVisitors().put(JavaxEjbInitVisitor.TYPE, new JavaxEjbInitVisitor<E, D, C, M, F>(ejbJarMethodMetadata));

        // add @Remove
        getAnnotationVisitors().put(JavaxEjbRemoveVisitor.TYPE, new JavaxEjbRemoveVisitor<E, D, C, M, F>(ejbJarMethodMetadata));

        // add @TransactionAttribute
        getAnnotationVisitors().put(JavaxEjbTransactionAttributeVisitor.TYPE,
                new JavaxEjbTransactionAttributeVisitor<M>(ejbJarMethodMetadata));

        // add @PostActivate
        getAnnotationVisitors().put(JavaxEjbPostActivateVisitor.TYPE,
                new JavaxEjbPostActivateVisitor<E, D, C, M, F>(ejbJarMethodMetadata));
        // add @PrePassivate
        getAnnotationVisitors().put(JavaxEjbPrePassivateVisitor.TYPE,
                new JavaxEjbPrePassivateVisitor<E, D, C, M, F>(ejbJarMethodMetadata));

        // add @Timeout
        getAnnotationVisitors().put(JavaxEjbTimeoutVisitor.TYPE, new JavaxEjbTimeoutVisitor<E, D, C, M, F>(ejbJarMethodMetadata));

        // add @AroundInvoke
        getAnnotationVisitors().put(JavaxInterceptorAroundInvokeVisitor.TYPE,
                new JavaxInterceptorAroundInvokeVisitor<E, D, C, M, F>(ejbJarMethodMetadata));

        // add @Interceptors
        getAnnotationVisitors().put(JavaxInterceptorInterceptorsVisitor.TYPE,
                new JavaxInterceptorInterceptorsVisitor<M>(ejbJarMethodMetadata));

        // add @ExcludeClassInterceptors
        getAnnotationVisitors().put(JavaxInterceptorExcludeClassInterceptorsVisitor.TYPE,
                new JavaxInterceptorExcludeClassInterceptorsVisitor<E, D, C, M, F>(ejbJarMethodMetadata));

        // add @RolesAllowed
        getAnnotationVisitors().put(JavaxAnnotationSecurityRolesAllowedVisitor.TYPE,
                new JavaxAnnotationSecurityRolesAllowedVisitor<M>(ejbJarMethodMetadata));

        // add @PermitAll
        getAnnotationVisitors().put(JavaxAnnotationSecurityPermitAllVisitor.TYPE,
                new JavaxAnnotationSecurityPermitAllVisitor<M>(ejbJarMethodMetadata));

        // add @DenyAll
        getAnnotationVisitors().put(JavaxAnnotationSecurityDenyAllVisitor.TYPE,
                new JavaxAnnotationSecurityDenyAllVisitor<E, D, C, M, F>(ejbJarMethodMetadata));

        // add @ExcludeDefaultInterceptors
        getAnnotationVisitors().put(JavaxInterceptorExcludeDefaultInterceptorsVisitor.TYPE,
                new JavaxInterceptorExcludeDefaultInterceptorsVisitor<M>(ejbJarMethodMetadata));

        // add @WebMethod
        getAnnotationVisitors().put(JavaxJwsWebMethodVisitor.TYPE,
                new JavaxJwsWebMethodVisitor<E, D, C, M, F>(ejbJarMethodMetadata));

        // add @Lock
        getAnnotationVisitors().put(JavaxEjbLockVisitor.TYPE,
                new JavaxEjbLockVisitor<M>(ejbJarMethodMetadata));
        
        // add @AccessTimeout
        getAnnotationVisitors().put(JavaxEjbAccessTimeoutVisitor.TYPE,
            new JavaxEjbAccessTimeoutVisitor<M>(ejbJarMethodMetadata));
        
        // add @AfterBegin
        getAnnotationVisitors().put(JavaxEjbAfterBeginVisitor.TYPE,
            new JavaxEjbAfterBeginVisitor<E, D, C, M, F>(ejbJarMethodMetadata));
        
        // add @BeforeCompletion
        getAnnotationVisitors().put(JavaxEjbBeforeCompletionVisitor.TYPE,
            new JavaxEjbBeforeCompletionVisitor<E, D, C, M, F>(ejbJarMethodMetadata));
        
        // add @AfterCompletion
        getAnnotationVisitors().put(JavaxEjbAfterCompletionVisitor.TYPE,
            new JavaxEjbAfterCompletionVisitor<E, D, C, M, F>(ejbJarMethodMetadata));        
        
        // add @Schedule
        getAnnotationVisitors().put(JavaxEjbScheduleVisitor.TYPE,
            new JavaxEjbScheduleVisitor<M>(ejbJarMethodMetadata));        

        // add @Schedules
        getAnnotationVisitors().put(JavaxEjbSchedulesVisitor.TYPE,
            new JavaxEjbSchedulesVisitor(ejbJarMethodMetadata));        
        
    }

}
