/**
 * EasyBeans
 * Copyright (C) 2012 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JavaxEjbSchedulesVisitor.java 6271 2012-07-10 11:10:15Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.method;

import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarMethodMetadata;
import org.ow2.util.ee.metadata.ejbjar.impl.struct.JEJBSchedule;
import org.ow2.util.scan.api.IType;

/**
 * This class manages the handling of &#64;{@link javax.ejb.Schedules} annotation.
 * @author Florent Benoit
 */
public class JavaxEjbSchedulesVisitor extends JavaxEjbScheduleVisitor<IEjbJarMethodMetadata<?, ?, ?, ?, ?>> {

    /**
     * Type of annotation.
     */
    public static final String TYPE = "Ljavax/ejb/Schedules;";

    /**
     * Object is added to the list.
     */
    private boolean isAdded = false;

    /**
     * Constructor.
     * @param annotationMetadata linked to a method metadata
     */
    public JavaxEjbSchedulesVisitor(final IEjbJarMethodMetadata<?, ?, ?, ?, ?> annotationMetadata) {
        super(annotationMetadata);
    }

    /**
     * Visits a primitive value of the annotation.<br>
     * @param name the value name.
     * @param value the actual value, whose type must be {@link Byte},
     *        {@link Boolean}, {@link Character}, {@link Short},
     *        {@link Integer}, {@link Long}, {@link Float}, {@link Double},
     *        {@link String} or {@link IType}.
     */
    @Override
    public void visit(final String name, final Object value) {
        if (isAdded) {
            setJEJBSchedule(new JEJBSchedule());
            isAdded = false;
        }

        // do super code
        super.visit(name, value);
    }

    /**
     * Visits the end of the annotation. <br>
     * Creates the object and store it.
     */
    @Override
    public void visitEnd() {
        // add object in the list
        if (!isAdded) {
            getAnnotationMetadata().addJavaxEjbSchedule(getJEJBSchedule());
            isAdded = true;
        }

    }

    /**
     * @return type of the annotation (its description)
     */
    @Override
    public String getType() {
        return TYPE;
    }

}
