/**
 * OW2 Util
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: DefaultEjbJarClassMetadataConfigurator.java 4389 2008-12-15 13:48:57Z alitokmen $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.ejbjar.impl.internal;

import org.ow2.util.ee.deploy.api.deployable.EJBDeployable;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.EjbJarClassMetadataConfigurator;
import org.ow2.util.scan.api.configurator.IFieldConfigurator;
import org.ow2.util.scan.api.configurator.IMethodConfigurator;
import org.ow2.util.scan.api.metadata.structures.JField;
import org.ow2.util.scan.api.metadata.structures.JMethod;

/**
 * Default configurator.
 * @param <E> an implementation of EJBDeployable
 * @author Florent Benoit
 */
public class DefaultEjbJarClassMetadataConfigurator<E extends EJBDeployable<E>>
        extends
        EjbJarClassMetadataConfigurator<E, DefaultEjbJarDeployableMetadata<E>, DefaultEjbJarClassMetadata<E>, DefaultEjbJarMethodMetadata<E>, DefaultEjbJarFieldMetadata<E>> {

    public DefaultEjbJarClassMetadataConfigurator(final DefaultEjbJarClassMetadata<E> classMetadata,
            final DefaultEjbJarDeployableMetadata<E> ejbJarArchiveMetadata, final boolean annotationParsingDesactived) {
        super(classMetadata, ejbJarArchiveMetadata, annotationParsingDesactived);
    }

    @Override
    protected IFieldConfigurator createFieldMetadataConfigurator(final JField field,
            final DefaultEjbJarClassMetadata<E> classMetadata, final boolean annotationParsingDesactived) {
        DefaultEjbJarFieldMetadata<E> fieldMetadata = new DefaultEjbJarFieldMetadata<E>(field, classMetadata);
        return new DefaultEjbJarFieldMetadataConfigurator<E>(fieldMetadata, annotationParsingDesactived);
    }

    @Override
    protected IMethodConfigurator createMethodMetadataConfigurator(final JMethod method,
            final DefaultEjbJarClassMetadata<E> classMetadata, final boolean annotationParsingDesactived) {
        DefaultEjbJarMethodMetadata<E> methodMetadata = new DefaultEjbJarMethodMetadata<E>(method, classMetadata);
        return new DefaultEjbJarMethodMetadataConfigurator<E>(methodMetadata, annotationParsingDesactived);
    }

}
